
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "crc_data.h"


// this example shows a simple routing in the following manner:
//
// CAN1  <--->  CAN2
// CAN3  <--->  CAN4
//



// identifier is needed by PCANFlash.exe -> do not delete
const  b8_t  Ident[] __attribute__((used)) = { "PCAN-Router_Pro" };


// info data for PCANFlash.exe
const crc_array_t  C2F_Array __attribute__((section(".C2F_Info"), used)) = {

	.Str = CRC_IDENT_STRING,
	.Version = 1 << 4 | 0,
	.Day = 20,
	.Month = 4,
	.Year = 12,
	.Mode = 1,

	// crc infos are patched during link time by flash.ld
	// crc value is patched by PCANFlash.exe
};


// variables for LED toggle
static u8_t LED_toggleCAN1;
static u8_t LED_toggleCAN2;
static u8_t LED_toggleCAN3;
static u8_t LED_toggleCAN4;



// main_greeting()
// transmitt a message at module start
static void  main_greeting ( void)
{
	CANMsg_t  Msg;
	

	Msg.Id   = 0x123;
	Msg.Len  = 8;
	Msg.Type = CAN_MSG_STANDARD;
	
	Msg.Data32[0] = 0x67452301;
	Msg.Data32[1] = 0xEFCDAB89;

	// Send Msg
	CAN_UserWrite ( CAN_BUS1, &Msg);
}




// main()
int main ( void)
{

	// init hardware
	HW_Init();
	
	
	// init CAN
	CAN_UserInit();
	
	
	// Set green LEDs for CAN1 to CAN4
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN3, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN4, HW_LED_GREEN);
	
	
	// send the greeting message
	main_greeting();
	
	
	// main loop
	while ( 1)
	{
		CANMsg_t  RxMsg, TxMsg;
		
		
		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) != 0)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_OFF);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// copy message
			TxMsg.Id   = RxMsg.Id;
			TxMsg.Type = RxMsg.Type;
			TxMsg.Len  = RxMsg.Len;
			
			TxMsg.Data32[0]	= RxMsg.Data32[0];
			TxMsg.Data32[1]	= RxMsg.Data32[1];
				
			// send
			CAN_UserWrite ( CAN_BUS2, &TxMsg);
		}


		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) != 0)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_OFF);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			
			// copy message
			TxMsg.Id   = RxMsg.Id;
			TxMsg.Type = RxMsg.Type;
			TxMsg.Len  = RxMsg.Len;
			
			TxMsg.Data32[0]	= RxMsg.Data32[0];
			TxMsg.Data32[1]	= RxMsg.Data32[1];
			
			// send
			CAN_UserWrite ( CAN_BUS1, &TxMsg);
		}
		
		
		// process messages from CAN3
		if ( CAN_UserRead ( CAN_BUS3, &RxMsg) != 0)
		{
			// message received from CAN3
			LED_toggleCAN3 ^= 1;

			if ( LED_toggleCAN3)
			{
				HW_SetLED ( HW_LED_CAN3, HW_LED_OFF);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN3, HW_LED_GREEN);
			}
			
			
			// copy message
			TxMsg.Id   = RxMsg.Id;
			TxMsg.Type = RxMsg.Type;
			TxMsg.Len  = RxMsg.Len;
			
			TxMsg.Data32[0]	= RxMsg.Data32[0];
			TxMsg.Data32[1]	= RxMsg.Data32[1];
			
			// send
			CAN_UserWrite ( CAN_BUS4, &TxMsg);
		}
		
		
		// process messages from CAN4
		if ( CAN_UserRead ( CAN_BUS4, &RxMsg) != 0)
		{
			// message received from CAN4
			LED_toggleCAN4 ^= 1;

			if ( LED_toggleCAN4)
			{
				HW_SetLED ( HW_LED_CAN4, HW_LED_OFF);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN4, HW_LED_GREEN);
			}
			
			
			// copy message
			TxMsg.Id   = RxMsg.Id;
			TxMsg.Type = RxMsg.Type;
			TxMsg.Len  = RxMsg.Len;
			
			TxMsg.Data32[0]	= RxMsg.Data32[0];
			TxMsg.Data32[1]	= RxMsg.Data32[1];
			
			// send
			CAN_UserWrite ( CAN_BUS3, &TxMsg);
		}
	}
}


